{% extends 'registration/registration_base.html' %}

{% block title %}Activated{% endblock %}

{% block registration_block %}
<p>
  Welcome to Orchestra!
</p>
<p>
  <a href="{% url 'orchestra:index' %}">Click here to launch the dashboard.</a>
</p>
{% endblock registration_block %}
