/**
 * Common stylesheet for Orchestra.
 *
 * This partial provides common variables and mixins useful for various view and
 * component styles.
 */
.disable-select {
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -khtml-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none; }

.row.no-padding, .row.no-padding > [class*="col-"] {
  padding-left: 0;
  padding-right: 0; }

.website-iframe {
  width: 100%;
  border: 1px solid #cccccc; }
