.orchestra-quill-wrapper {
  background-color: #fff;
  border-left: 1px solid #ccc;
  border-right: 1px solid #ccc;
  box-shadow: 0px 0px 10px #ccc;
  margin-left: auto;
  margin-right: auto;
  width: 100%;

  .orchestra-quill-editor {
    height: 400px;
  }

  .orchestra-quill-toolbar {
    margin-top: 15px;
    border-bottom: 1px solid #ccc;
    border-top: 1px solid #ccc;
  }
}
