{% extends 'registration/registration_base.html' %}

{% block title %}Password reset{% endblock %}

{% block registration_block %}
    <p>An email with password reset instructions has been sent.</p>
{% endblock registration_block %}
